# -*- coding: utf-8 -*-
# Author: allen
# Copyright (c) 2020 RuiCare All rights reserved.
import os
import xlwt
import datetime
from core import configer
from util import dcmutil

config = configer.get_config(False, True)
dcm_dir = config.get('DcmDir')
roi_name_affix = config.get('RoiNameAffix')


def get_first_value_from_dict(dist):
    for key, value in dist.items():
        return value


def get_roi_names(pat_dir):
    roi_names = []
    dcm_tree = dcmutil.get_dcm_tree(pat_dir)
    for patId, pat_dict in dcm_tree.items():
        for study_uid, study_dict in pat_dict.items():
            if not study_dict.get('CT'): return roi_names
            if not study_dict.get('RTSTRUCT'): return roi_names
            rs_dcm_data_dict = dcmutil.get_dcm_data_dict(study_dict.get('RTSTRUCT'))
            for rs_uid, rs_dcm_data in rs_dcm_data_dict.items():
                roi_contour_sequences = rs_dcm_data.get('ROIContourSequence')
                if roi_contour_sequences is None: return roi_names
                for roi_contour_sequence in roi_contour_sequences:
                    contours_sequences = roi_contour_sequence.get('ContourSequence')
                    if contours_sequences is None: continue
                    roi_number = int(roi_contour_sequence.get('ReferencedROINumber'))
                    roi_name = None
                    for roi_sequence in rs_dcm_data.get('StructureSetROISequence'):
                        if roi_sequence.get('ROINumber') == roi_number:
                            roi_name = roi_sequence.get('ROIName')
                            break
                    if roi_name is None: continue

                    for roi_name_key, affix_list in roi_name_affix.items():
                        for affix in affix_list:
                            if roi_name.lower().__contains__(affix):
                                roi_names.append(roi_name)
    return roi_names


def split_array(arr, size):
    s = []
    for i in range(0, int(len(arr)) + 1, size):
        c = arr[i:i + size]
        s.append(c)
    return s


def start():
    workbook = xlwt.Workbook(encoding='utf-8')
    worksheet = workbook.add_sheet(datetime.datetime.now().strftime("%Y%m%d%H%M%S"))

    pat_dir_names = os.listdir(dcm_dir)
    pat_dir_names.sort()
    current_rows = 0
    for row, pat_dir_name in enumerate(pat_dir_names):

        pat_dir = os.path.join(dcm_dir, pat_dir_name)
        roi_names = []
        try:
            roi_names = get_roi_names(pat_dir)
            print(f'The program is extracting roi names of pat:{pat_dir_name}')
        except Exception as e:
            print('Extracting roi names %s catch error:%s' % (pat_dir, e))
        roi_names.sort()

        roi_names_list = split_array(roi_names, 10)
        for roi_names in roi_names_list:
            worksheet.write_merge(current_rows, current_rows + 1, 0, 0, pat_dir_name)

            for i, roi_name in enumerate(roi_names):
                worksheet.write(current_rows, i + 1, label=roi_name)
                #workbook.save('%s/ROI.xls' % configer.get_dir())

            current_rows += 2

    workbook.save('%s/ROI.xls' % configer.get_dir())
