# -*- coding: utf-8 -*-
# Author: allen
# Copyright (c) 2020 RuiCare All rights reserved.
import json
import os
import xlwt
import datetime
from core import configer
from util import dcmutil, fileutil

config = configer.get_config(False, True)
dcm_dir = config.get('DcmDir')
roi_name_affix = config.get('RoiNameAffix')


def get_first_value_from_dict(dist):
    for key, value in dist.items():
        return value


def get_roi_names(pat_dir):
    roi_names = []
    dcm_tree = dcmutil.get_dcm_tree(pat_dir)
    for patId, pat_dict in dcm_tree.items():
        for study_uid, study_dict in pat_dict.items():
            if not study_dict.get('CT'): return roi_names
            if not study_dict.get('RTSTRUCT'): return roi_names
            rs_dcm_data_dict = dcmutil.get_dcm_data_dict(study_dict.get('RTSTRUCT'))
            for rs_uid, rs_dcm_data in rs_dcm_data_dict.items():
                roi_contour_sequences = rs_dcm_data.get('ROIContourSequence')
                if roi_contour_sequences is None: return roi_names
                for roi_contour_sequence in roi_contour_sequences:
                    contours_sequences = roi_contour_sequence.get('ContourSequence')
                    if contours_sequences is None: continue
                    roi_number = int(roi_contour_sequence.get('ReferencedROINumber'))
                    roi_name = None
                    for roi_sequence in rs_dcm_data.get('StructureSetROISequence'):
                        if roi_sequence.get('ROINumber') == roi_number:
                            roi_name = roi_sequence.get('ROIName')
                            break
                    if roi_name is None: continue

                    for roi_name_key, roi_config in roi_name_affix.items():
                        if is_needed_convert_roi(roi_name, roi_config):
                            roi_names.append(roi_name)

    return roi_names


def split_array(arr, size):
    s = []
    for i in range(0, int(len(arr)) + 1, size):
        c = arr[i:i + size]
        s.append(c)
    return s


def is_needed_convert_roi(label_name, roi_config, needed_converted_rois=-1):
    contains = roi_config.get('Contains')
    uncontains = roi_config.get('UnContains')
    uncontains = uncontains if uncontains else []

    is_contain = False
    for contain in contains:
        if label_name.lower().__contains__(contain): is_contain = True

    is_uncontain = True
    for unContain in uncontains:
        if label_name.lower().__contains__(unContain): is_uncontain = False

    if needed_converted_rois == -1:
        return is_contain and is_uncontain
    else:
        return is_contain and is_uncontain and (needed_converted_rois and label_name in needed_converted_rois)


def start():
    result = {}

    pat_dir_names = os.listdir(dcm_dir)
    pat_dir_names.sort()
    for row, pat_dir_name in enumerate(pat_dir_names):

        pat_dir = os.path.join(dcm_dir, pat_dir_name)
        roi_names = []
        try:
            roi_names = get_roi_names(pat_dir)
        except Exception as e:
            print('Extracting roi names %s catch error:%s' % (pat_dir, e))

        print(pat_dir_name, roi_names)

        result[pat_dir_name] = roi_names

    fileutil.write('./RoiNames.json', json.dumps(result))
