# -*- coding: utf-8 -*-
# Author: allen
# Copyright (c) 2020 RuiCare All rights reserved.
import os

import pydicom

from util import fileutil


def get_dcm_data(file):
    dcm_data = None
    try:
        dcm_data = pydicom.dcmread(file, force=True)
        if dcm_data is not None and dcm_data.get("Modality") is not None:
            return dcm_data
    except Exception as e:
        print(e)
    return dcm_data


def get_dcm_tree(dcm_dir):
    dcm_tree_list = {}
    file_path_list = fileutil.list_all_file_path(dcm_dir)
    for file_path in file_path_list:
        if file_path.endswith('.DS_Store') or file_path.startswith('.'): continue
        dcm_data = get_dcm_data(file_path)
        if dcm_data is None: continue
        dcm_type = dcm_data.get('Modality')
        if dcm_type is not None and dcm_type in ['CT', 'RTSTRUCT']:
            # 获取基本信息
            modality = dcm_data.get('Modality')
            patient_id = dcm_data.get('PatientID')
            study_instance_uid = dcm_data.get('StudyInstanceUID')
            series_instance_uid = dcm_data.get('SeriesInstanceUID')
            sop_instance_uid = dcm_data.get('SOPInstanceUID')
            # 归类存储
            if dcm_tree_list.get(study_instance_uid) is None:
                dcm_tree_list[study_instance_uid] = {}
            referenced_ct_series_instance_uid = None
            if dcm_type == 'CT':
                referenced_ct_series_instance_uid = series_instance_uid
            if modality == 'RTSTRUCT':
                referenced_ct_series_instance_uid = \
                    dcm_data.get('ReferencedFrameOfReferenceSequence')[0].get('RTReferencedStudySequence')[0].get(
                        'RTReferencedSeriesSequence')[0].get('SeriesInstanceUID')
            if dcm_tree_list.get(study_instance_uid).get(referenced_ct_series_instance_uid) is None:
                dcm_tree_list[study_instance_uid][referenced_ct_series_instance_uid] = {'CT': {}, 'RTSTRUCT': {}}
            dcm_tree_list[study_instance_uid][referenced_ct_series_instance_uid][dcm_type][sop_instance_uid] = file_path
    return dcm_tree_list


def get_dcm_data_dict(dcm_path_dict):
    dcm_data_dict = {}
    for sop_instance_uid, dcm_path in dcm_path_dict.items():
        dcm_data_dict[sop_instance_uid] = get_dcm_data(dcm_path)
    return dcm_data_dict


def get_rs_dcm(dcm_pat_dir):
    dcm_paths = fileutil.list_all_file_path(dcm_pat_dir)
    rs_dcm_datas, rs_dcm_paths = [], []
    for dcm_path in dcm_paths:
        dcm_data = get_dcm_data(dcm_path)
        md = dcm_data.get("Modality")
        if md is not None and md == 'RTSTRUCT':
            rs_dcm_datas.append(dcm_data)
            rs_dcm_paths.append(dcm_path)
    return rs_dcm_datas, rs_dcm_paths


def get_newest_rs(dcm_pat_dir):
    _, rs_dcm_paths = get_rs_dcm(dcm_pat_dir)
    rs_dcm_paths.sort(key=lambda x: os.path.getmtime(x))
    return (get_dcm_data(rs_dcm_paths[0]), rs_dcm_paths[0]) if rs_dcm_paths else (None, None)


def get_roi_data(rs_dcm_data, with_contour=True, roi_numbers=None):
    roi_data = {}
    roi_contour_sequences = rs_dcm_data.get('ROIContourSequence')
    if roi_contour_sequences is None: return roi_data
    for roi_contour_sequence in roi_contour_sequences:
        contour_sequences = roi_contour_sequence.get('ContourSequence')
        if contour_sequences is None: continue
        roi_number = int(roi_contour_sequence.get('ReferencedROINumber'))
        if roi_numbers and roi_number not in roi_numbers: continue
        display_color = roi_contour_sequence.get('ROIDisplayColor')
        roi_color = list(display_color) if display_color else None
        roi_name, roi_generation_algorithm = None, None

        for roi_sequence in rs_dcm_data.get('StructureSetROISequence'):
            if roi_sequence.get('ROINumber') == roi_number:
                roi_name = roi_sequence.get('ROIName')
                roi_generation_algorithm = roi_sequence.get('ROIGenerationAlgorithm')
                break
        if roi_name.__contains__('[') or roi_name.__contains__(']'): continue

        roi_contour = {}
        if with_contour:
            for contour_sequence in contour_sequences:
                contour_data = contour_sequence.get('ContourData')
                current_z = '{:.2f}'.format(float(contour_data[-1]))
                if roi_contour.get(current_z) is None: roi_contour[current_z] = []
                roi_contour[current_z].append(list(contour_data))
        roi_data[roi_name] = {'roi_number': roi_number, 'roi_color': roi_color, 'roi_name': roi_name,
                              'roi_generation_algorithm': roi_generation_algorithm}
        if with_contour:
            roi_data[roi_name]['roi_contour'] = roi_contour

    return roi_data


def get_roi_info(rs_dcm_data):
    return get_roi_data(rs_dcm_data, with_contour=False)
